"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createCorrectResponseSession = void 0;
exports.createDefaultModel = createDefaultModel;
exports.getScore = void 0;
exports.model = model;
exports.normalize = void 0;
exports.outcome = outcome;
exports.validate = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _isEmpty = _interopRequireDefault(require("lodash/isEmpty"));

var _utils = require("./utils");

var _defaults = _interopRequireDefault(require("./defaults"));

var _controllerUtils = require("@pie-lib/pie-toolbox/controller-utils");

var _excluded = ["verticalMode", "choicesLayout"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

var prepareChoice = function prepareChoice(model, env, defaultFeedback) {
  return function (choice) {
    var _ref = env || {},
        role = _ref.role,
        mode = _ref.mode;

    var out = {
      label: choice.label,
      value: choice.value
    };

    if (role === 'instructor' && (mode === 'view' || mode === 'evaluate')) {
      out.rationale = model.rationaleEnabled ? choice.rationale : null;
    } else {
      out.rationale = null;
    }

    if (mode === 'evaluate') {
      out.correct = !!choice.correct;

      if (model.feedbackEnabled) {
        var feedbackType = choice.feedback && choice.feedback.type || 'none';

        if (feedbackType === 'default') {
          out.feedback = defaultFeedback[choice.correct ? 'correct' : 'incorrect'];
        } else if (feedbackType === 'custom') {
          out.feedback = choice.feedback.value;
        }
      }
    }

    return out;
  };
};

function createDefaultModel() {
  var model = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return new Promise(function (resolve) {
    return resolve(_objectSpread(_objectSpread({}, _defaults["default"]), model));
  });
}

var normalize = function normalize(question) {
  var _ref2 = question || {},
      verticalMode = _ref2.verticalMode,
      choicesLayout = _ref2.choicesLayout,
      questionProps = (0, _objectWithoutProperties2["default"])(_ref2, _excluded);

  return _objectSpread(_objectSpread(_objectSpread({}, _defaults["default"]), questionProps), {}, {
    // This is used for offering support for old models which have the property verticalMode
    // Same thing is set in authoring : packages/multiple-choice/configure/src/index.jsx - createDefaultModel
    choicesLayout: choicesLayout || verticalMode === false && 'horizontal' || _defaults["default"].choicesLayout
  });
};
/**
 *
 * @param {*} question
 * @param {*} session
 * @param {*} env
 * @param {*} updateSession - optional - a function that will set the properties passed into it on the session.
 */


exports.normalize = normalize;

function model(_x, _x2, _x3, _x4) {
  return _model.apply(this, arguments);
}

function _model() {
  _model = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(question, session, env, updateSession) {
    var normalizedQuestion, defaultFeedback, choices, lockChoiceOrder, out, _ref4, role, mode;

    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            normalizedQuestion = normalize(question);
            defaultFeedback = Object.assign({
              correct: 'Correct',
              incorrect: 'Incorrect'
            }, normalizedQuestion.defaultFeedback);
            choices = (normalizedQuestion.choices || []).map(prepareChoice(normalizedQuestion, env, defaultFeedback));
            lockChoiceOrder = (0, _controllerUtils.lockChoices)(normalizedQuestion, session, env);

            if (lockChoiceOrder) {
              _context.next = 8;
              break;
            }

            _context.next = 7;
            return (0, _controllerUtils.getShuffledChoices)(choices, session, updateSession, 'value');

          case 7:
            choices = _context.sent;

          case 8:
            out = {
              disabled: env.mode !== 'gather',
              mode: env.mode,
              prompt: normalizedQuestion.promptEnabled ? normalizedQuestion.prompt : null,
              choicesLayout: normalizedQuestion.choicesLayout,
              gridColumns: normalizedQuestion.gridColumns,
              choiceMode: normalizedQuestion.choiceMode,
              keyMode: normalizedQuestion.choicePrefix,
              choices: choices,
              responseCorrect: env.mode === 'evaluate' ? (0, _utils.isResponseCorrect)(normalizedQuestion, session) : undefined,
              language: normalizedQuestion.language,
              extraCSSRules: normalizedQuestion.extraCSSRules,
              fontSizeFactor: normalizedQuestion.fontSizeFactor,
              isSelectionButtonBelow: normalizedQuestion.isSelectionButtonBelow,
              selectedAnswerBackgroundColor: normalizedQuestion.selectedAnswerBackgroundColor || 'initial',
              minSelections: normalizedQuestion.minSelections,
              maxSelections: normalizedQuestion.maxSelections,
              keyboardEventsEnabled: normalizedQuestion.keyboardEventsEnabled,
              autoplayAudioEnabled: normalizedQuestion.autoplayAudioEnabled,
              completeAudioEnabled: normalizedQuestion.completeAudioEnabled,
              customAudioButton: normalizedQuestion.customAudioButton
            };
            _ref4 = env || {}, role = _ref4.role, mode = _ref4.mode;

            if (role === 'instructor' && (mode === 'view' || mode === 'evaluate')) {
              out.teacherInstructions = normalizedQuestion.teacherInstructionsEnabled ? normalizedQuestion.teacherInstructions : null;
            } else {
              out.teacherInstructions = null;
            }

            return _context.abrupt("return", out);

          case 12:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _model.apply(this, arguments);
}

var getScore = function getScore(config, session) {
  if (!session || (0, _isEmpty["default"])(session)) {
    return 0;
  }

  var selectedChoices = session.value || [];
  var correctChoices = (config.choices || []).filter(function (ch) {
    return ch.correct;
  });
  var score = selectedChoices.reduce(function (acc, selectedChoice) {
    return acc + (correctChoices.find(function (ch) {
      return ch.value === selectedChoice;
    }) ? 1 : 0);
  }, 0);

  if (correctChoices.length < selectedChoices.length) {
    score -= selectedChoices.length - correctChoices.length;

    if (score < 0) {
      score = 0;
    }
  }

  var str = correctChoices.length ? score / correctChoices.length : 0;
  return parseFloat(str.toFixed(2));
};
/**
 *
 * The score is partial by default for checkbox mode, allOrNothing for radio mode.
 * To disable partial scoring for checkbox mode you either set model.partialScoring = false or env.partialScoring = false. the value in `env` will
 * override the value in `model`.
 * @param {Object} model - the main model
 * @param {*} session
 * @param {Object} env
 */


exports.getScore = getScore;

function outcome(model, session, env) {
  return new Promise(function (resolve) {
    if (!session || (0, _isEmpty["default"])(session)) {
      resolve({
        score: 0,
        empty: true
      });
    } else {
      var partialScoringEnabled = _controllerUtils.partialScoring.enabled(model, env) && model.choiceMode !== 'radio';
      var score = getScore(model, session);
      resolve({
        score: partialScoringEnabled ? score : score === 1 ? 1 : 0,
        empty: false
      });
    }
  });
}

var createCorrectResponseSession = function createCorrectResponseSession(question, env) {
  return new Promise(function (resolve) {
    if (env.mode !== 'evaluate' && env.role === 'instructor') {
      var _ref3 = question || {
        choices: []
      },
          choices = _ref3.choices;

      resolve({
        id: '1',
        value: choices.filter(function (c) {
          return c.correct;
        }).map(function (c) {
          return c.value;
        })
      });
    } else {
      resolve(null);
    }
  });
}; // remove all html tags


exports.createCorrectResponseSession = createCorrectResponseSession;

var getInnerText = function getInnerText(html) {
  return (html || '').replaceAll(/<[^>]*>/g, '');
}; // remove all html tags except img, iframe and source tag for audio


var getContent = function getContent(html) {
  return (html || '').replace(/(<(?!img|iframe|source)([^>]+)>)/gi, '');
};

var validate = function validate() {
  var model = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var choices = model.choices;
  var _config$minAnswerChoi = config.minAnswerChoices,
      minAnswerChoices = _config$minAnswerChoi === void 0 ? 2 : _config$minAnswerChoi,
      maxAnswerChoices = config.maxAnswerChoices;
  var reversedChoices = (0, _toConsumableArray2["default"])(choices || []).reverse();
  var choicesErrors = {};
  var rationaleErrors = {};
  var errors = {};
  ['teacherInstructions', 'prompt'].forEach(function (field) {
    var _config$field;

    if ((_config$field = config[field]) !== null && _config$field !== void 0 && _config$field.required && !getContent(model[field])) {
      errors[field] = 'This field is required.';
    }
  });
  var hasCorrectResponse = false;
  reversedChoices.forEach(function (choice, index) {
    var _config$rationale;

    var correct = choice.correct,
        value = choice.value,
        label = choice.label,
        rationale = choice.rationale;

    if (correct) {
      hasCorrectResponse = true;
    }

    if (!getContent(label)) {
      choicesErrors[value] = 'Content should not be empty.';
    } else {
      var identicalAnswer = reversedChoices.slice(index + 1).some(function (c) {
        return c.label === label;
      });

      if (identicalAnswer) {
        choicesErrors[value] = 'Content should be unique.';
      }
    }

    if ((_config$rationale = config.rationale) !== null && _config$rationale !== void 0 && _config$rationale.required && !getContent(rationale)) {
      rationaleErrors[value] = 'This field is required.';
    }
  });
  var nbOfChoices = (choices || []).length;

  if (nbOfChoices < minAnswerChoices) {
    errors.answerChoices = "There should be at least ".concat(minAnswerChoices, " choices defined.");
  } else if (nbOfChoices > maxAnswerChoices) {
    errors.answerChoices = "No more than ".concat(maxAnswerChoices, " choices should be defined.");
  }

  if (!hasCorrectResponse) {
    errors.correctResponse = 'No correct response defined.';
  }

  if (!(0, _isEmpty["default"])(choicesErrors)) {
    errors.choices = choicesErrors;
  }

  if (!(0, _isEmpty["default"])(rationaleErrors)) {
    errors.rationale = rationaleErrors;
  }

  return errors;
};

exports.validate = validate;
//# sourceMappingURL=index.js.map